% Recurrent Neural Equalizer initialization. 
%
% Syntax: RNEinit
backup;
%% Transmit signal generation and modulation:

fc = 5e9;                           % Carrier wave frequency [Hz].
mod = 16;                           % Modulation order. %%BPSK da shit?
n_S = 1e4;                          % Number of training symbols (iterations).

%% FGN Channel model:

c = 3e8;                                        % Speed of light, ish [m/s].
fc = 5e9;                                       % Carrier wave frequency [Hz].
K_rice = 5.6234;                                % Rice factor. Typical value is 15 dB.
v = 25;                                         % Air speed [m/s]. Typical value for small UAV.
D = 1e4;                                        % Distance between antennas, in metres.
n_paths = 5;                                    % Number of descrete echo paths, first path is rician fading process with k factor of K_rice, while the rest are independent Rayleigh fading processes.

lambda = c/fc;                                  % Wavelength of carrier wave.
fd = v/lambda;                                  % Maximum doppler shift (UAV attitude is perpendicular to ground antenna direction).
delay_E = D/c;                                  % Echo path time delay, expected value for air-ground links.
tau = (raylrnd(delay_E,n_paths,1))';            % Rayleight random distributed echo path delay times, first value corresponds to LOS path.
ts = (min(tau(tau~=0)))/2;                      % Sample time of the input signal, in seconds.
pdb = zeros(1,length(tau));                     % Average path gains.
chan = ricianchan(ts,fd,K_rice,tau,pdb);        % Constructs a frequency-selective (multiple paths) fading-channel object.

chan.StoreHistory = 1;                          % Store history to enable visualization
chan.DopplerSpectrum=[doppler.ajakes([-1 0.8])];% Use asymetric Jakes Doppler spectra

%% NARX Channel model related

c = [0.3482;0.8704;0.3482];         % Channel time-invarient coefficients.
chan_l = length(c);                	% Channel length.
fade = 0.1;                         % Fading rate parameter.
distortion = 0.2;                   % Non-linear distortion.
a = time_variant(length(c),1,fade); % Channel time-varient coefficients.
SNR = 10;                           % Signal to noise ratio.
k_n = SNR^-1;                       % Front end AWGN stdev.
% k_n = 0;


%% Neural Network related:

training = true;                  	% Training mode.
hard = false;                       % Hard decision.
N = 10;                             % Estimated impulse response, in sample periods.
d = 2;                              % Decision delay.
p = 3;                              % Number of hidden nodes.

m = d+1;                            % Length of feed-forward inputs.
n = N + m - d - 2;                  % Length of decision-feedback vector.
L = 2*(p*(m + n + p + 1) + p + 1);  % System dimensions (number of weights).
r = complex([-1 + 2*rand(m,1)],[-1 + 2*rand(m,1)]);          	% Initialize m-by-1 feed-forward input vector.
u = complex([-1 + 2*rand(n,1)],[-1 + 2*rand(n,1)]);          	% Initialize n-by-1 decision-feedback input vector.
x_re = -1 + 2*rand(p,1);            % Initialize p-by-1 hidden real state vector.
x_im = -1 + 2*rand(p,1);            % Initialize p-by-1 hidden imaginary state vector.
x_fb_re = -1 + 2*rand(p,1);         % Initialize p-by-1 feedback real state vector.
x_fb_im = -1 + 2*rand(p,1);         % Initialize p-by-1 feedback imaginary state vector.
z_re = [1 real(r)' real(u)' x_fb_re']'; 
z_im = [1 imag(r)' imag(u)' x_fb_im']';                   % Augment neural net input vector.
% yd = [-1 + 2*rand(d + 1,1)];       	% Initialize desired neural net output vector.
y = [0;0];                          % Neural net real output.
yc = complex(0, 0);                 % Convert output to complex.
y_mean = [0;0];
s_hat = complex(rand, rand);        % Initialize decision vector.

%% Unscented Kalman filter related:

alpha = 0.5;                        % Corresponds to sigma point spreading.
beta = 2;                           % Scaling parameter for distribution skewness. 2 is optimal for Gaussian.           
kappa = 0;                       	% Corresponds to sigma point spreading.
lambda = ((alpha^2)*(L+kappa))-L;  	% Scaling factor for unscented transform.
forget = 0.999;                     % Forgetting factor, for process-noise covariance.
k = sqrt(lambda + L);               % Sigma point weight parameter.
R = 2*eye(2);                       % Artificial measurement-noise (co)variance.

%w = w_backup(:,3);                  % Initialize weight vector and covariance.
w = [-0.5 + rand(L,1)];
P = diag(5*ones(1,length(w)));
% P = P_backup(:,L+1:2*L);

WM = zeros(2*L+1,1);                % Calculate sigma point weights.
WC = zeros(2*L+1,1);
for j=1:2*L+1
  if j==1
    wm = lambda / (L + lambda);
    wc = lambda / (L + lambda) + (1 - alpha^2 + beta);
  else
    wm = 1 / (2 * (L + lambda));
    wc = wm;
  end
  WM(j) = wm;
  WC(j) = wc;
end
W = eye(length(WC)) - repmat(WM,1,length(WM));
W = W * diag(WC) * W';

%% Debug related:
error = zeros(n_S,100);
i=0;
debug_eig = zeros(L,n_S);
e = 0;%zeros(n_S,1);
filtered_MSE = zeros(n_S,100);
unfiltered_MSE = zeros(n_S,100);
BER = zeros(n_S,3);
SER_y = zeros(100,2);
SER_r = zeros(100,2);
RNE_improvement = zeros(100,1);
log_e = zeros(n_S,1);
log_y = zeros(n_S,1);
log_s = zeros(n_S,1);
log_s_hat = zeros(n_S,1);
ref=qammod(linspace(0,mod-1,mod), mod);
nonEqualizedSER = zeros(20,1);
equalizedSER = zeros(20,1);
% Normal_Error = zeros(3,b);